# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0003_initial'),
        ('encounters', '0003_initial'),
        ('episodeofcares', '0002_initial'),
        ('groups', '0002_initial'),
        ('healthcareservices', '0001_initial'),
        ('immunizationrecommendations', '0002_initial'),
        ('locations', '0001_initial'),
        ('observations', '0002_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='encounterbasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_based_on_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='encounter_based_on', to='encounters.encounterbasedonreference'),
        ),
        migrations.AddField(
            model_name='encounterdiagnosis',
            name='condition',
            field=models.ManyToManyField(blank=True, related_name='encounter_diagnosis_condition', to='conditions.conditioncodeablereference'),
        ),
        migrations.AddField(
            model_name='encounterdiagnosis',
            name='use',
            field=models.ManyToManyField(blank=True, related_name='encounter_diagnosis_use', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='diagnosis',
            field=models.ManyToManyField(blank=True, related_name='encounter_diagnosis', to='encounters.encounterdiagnosis'),
        ),
        migrations.AddField(
            model_name='encounterepisodeofcarereference',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_episode_of_care_reference_encounter', to='encounters.encounter'),
        ),
        migrations.AddField(
            model_name='encounterepisodeofcarereference',
            name='episode_of_care',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_episode_of_care_reference_episode_of_care', to='episodeofcares.episodeofcare'),
        ),
        migrations.AddField(
            model_name='encounterepisodeofcarereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_episode_of_care_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='encounterlocation',
            name='form',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_location_form', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounterlocation',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_location_location', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='encounterlocation',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_location_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='location',
            field=models.ManyToManyField(blank=True, related_name='encounter_location', to='encounters.encounterlocation'),
        ),
        migrations.AddField(
            model_name='encounterparticipant',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='encounterparticipant',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='encounter_participant_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='participant',
            field=models.ManyToManyField(blank=True, related_name='encounter_participant', to='encounters.encounterparticipant'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='healthcareservice',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_healthcareservice', to='healthcareservices.healthcareservice'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='encounterparticipantactorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_participant_actor_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='encounterparticipant',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='participant_actor', to='encounters.encounterparticipantactorreference'),
        ),
        migrations.AddField(
            model_name='encounterreason',
            name='use',
            field=models.ManyToManyField(blank=True, related_name='encounter_reason_use', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='encounter_reason', to='encounters.encounterreason'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluecodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='encounterreason',
            name='value',
            field=models.ManyToManyField(blank=True, related_name='encounter_reason_value', to='encounters.encounterreasonvaluecodeablereference'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluereference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_value_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluereference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_value_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_value_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluereference',
            name='immunization_recommendation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_value_reference_immunization_recommendation', to='immunizationrecommendations.immunizationrecommendation'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluereference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_value_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluereference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_value_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='encounterreasonvaluecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='encounter_reason_codeable_reference_reference', to='encounters.encounterreasonvaluereference'),
        ),
        migrations.AddField(
            model_name='encounterreference',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='encounter_reference_encounter', to='encounters.encounter'),
        ),
        migrations.AddField(
            model_name='encounterreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='encounter_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='encounter',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='encounter_part_of', to='encounters.encounterreference'),
        ),
    ]



"""Personal Relationships views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import PersonalRelationship
from .serializers import PersonalRelationshipSerializer


class PersonalRelationshipListView(APIView):
    """Personal Relationship list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: PersonalRelationshipSerializer(many=True)})
    def get(self, request):
        """Get a list of personal relationships."""
        personal_relationships = PersonalRelationship.objects.all()
        serializer = PersonalRelationshipSerializer(personal_relationships, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=PersonalRelationshipSerializer,
        responses={201: PersonalRelationshipSerializer},
    )
    def post(self, request):
        """Create a personal relationship."""
        serializer = PersonalRelationshipSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data, status=status.HTTP_201_CREATED)


class PersonalRelationshipDetailView(APIView):
    """Personal Relationship detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a personal relationship object."""
        try:
            return PersonalRelationship.objects.get(pk=pk)
        except PersonalRelationship.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: PersonalRelationshipSerializer})
    def get(self, request, pk):
        """Get a personal relationship."""
        personal_relationship = self.get_object(pk)
        serializer = PersonalRelationshipSerializer(personal_relationship)
        return Response(serializer.data)

    @extend_schema(
        request=PersonalRelationshipSerializer,
        responses={200: PersonalRelationshipSerializer},
    )
    def put(self, request, pk):
        """Update a personal relationship."""
        personal_relationship = self.get_object(pk)
        serializer = PersonalRelationshipSerializer(
            personal_relationship, data=request.data
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a personal relationship."""
        personal_relationship = self.get_object(pk)
        personal_relationship.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Persons views."""

from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Person
from .serializers import PersonSerializer


class PersonListView(APIView):
    """Healthcare Service list view."""

    serializer = PersonSerializer
    permission_classes = [AllowAny]

    @extend_schema(responses={200, PersonSerializer(many=True)})
    def get(self, request):
        """Get all healthcare services."""
        persons = Person.objects.all()
        serializer = self.serializer(persons, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=PersonSerializer,
        responses={201, PersonSerializer},
    )
    def post(self, request):
        """Create a new healthcare service."""
        serializer = self.serializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class PersonDetailView(APIView):
    """Healthcare Service detail view."""

    serializer = PersonSerializer

    def get_object(self, pk=None):
        """Get healthcare service object."""
        try:
            return Person.objects.get(pk=pk)
        except Person.DoesNotExist:
            return Response(status=status.HTTP_404_NOT_FOUND)

    @extend_schema(responses={200, PersonSerializer})
    def get(self, request, pk):
        """Get a healthcare service."""
        person = self.get_object(pk)
        serializer = self.serializer(person)
        return Response(serializer.data)

    @extend_schema(responses={200, PersonSerializer})
    def patch(self, request, pk):
        """Update a healthcare service."""
        person = self.get_object(pk)
        serializer = self.serializer(person, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk):
        """Delete a healthcare service."""
        person = self.get_object(pk)
        person.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""organization affiliation serializers."""

from dfhir.base.serializers import (
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    ExtendedContactDetailSerializer,
    IdentifierSerializer,
    PeriodSerializer,
)
from dfhir.endpoints.serializers import EndpointReferenceSerializer
from dfhir.healthcareservices.serializers import HealthCareServiceReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.organizationaffiliations.models import OrganizationAffiliation
from dfhir.organizations.serializers import OrganizationReferenceSerializer


class OrganizationAffiliationSerializer(BaseWritableNestedModelSerializer):
    """organization affiliation serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    period = PeriodSerializer(required=False, many=False)
    organization = OrganizationReferenceSerializer(required=False, many=False)
    participating_organization = OrganizationReferenceSerializer(
        required=False, many=False
    )
    network = OrganizationReferenceSerializer(required=False, many=True)
    code = CodeableConceptSerializer(required=False, many=True)
    specialty = CodeableConceptSerializer(required=False, many=True)
    location = LocationReferenceSerializer(required=False, many=True)
    healthcare_service = HealthCareServiceReferenceSerializer(required=False, many=True)
    contact = ExtendedContactDetailSerializer(required=False, many=True)
    endpoint = EndpointReferenceSerializer(required=False, many=True)

    class Meta:
        """meta options."""

        model = OrganizationAffiliation
        exclude = ["created_at", "updated_at"]



"""Plandefinitions choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class PlanDefinitionStatus(models.TextChoices):
    """PlanDefinition status choices."""

    ACTIVE = "active", _("Active")
    DRAFT = "draft", _("Draft")
    RETIRED = "retired", _("Retired")
    UNKNOWN = "unknown", _("Unknown")


class PlanDefinitionActionPriority(models.TextChoices):
    """PlanDefinition action priority choices."""

    ROUTINE = "routine", _("Routine")
    URGENT = "urgent", _("Urgent")
    ASAP = "asap", _("As soon as possible")
    STAT = "stat", _("Stat")


class PlanDefinitionActionGroupingBehavior(models.TextChoices):
    """PlanDefinition action grouping behavior choices."""

    VISUAL_GROUP = "visual-group", _("Visual Group")
    LOGICAL_GROUP = "logical-group", _("Logical Group")
    SENTENCE_GROUP = "sentence-group", _("Sentence Group")


class PlanDefinitionActionSelectionBehavior(models.TextChoices):
    """PlanDefinition action selection behavior choices."""

    ANY = "any", _("Any")
    ALL = "all", _("All")
    ALL_OR_NONE = "all-or-none", _("All or None")
    EXACTLY_ONE = "exactly-one", _("Exactly One")
    AT_MOST_ONE = "at-most-one", _("At Most One")
    ONE_OR_MORE = "one-or-more", _("One or More")


class PlanDefinitionActionRequiredBehavior(models.TextChoices):
    """PlanDefinition action required behavior choices."""

    MUST = "must", _("Must")
    COULD = "could", _("Could")
    MUST_UNLESS_DOCUMENTED = "must-unless-documented", _("Must unless Documented")


class PlanDefinitionActionPrecheckBehavior(models.TextChoices):
    """PlanDefinition action precheck behavior choices."""

    YES = "yes", _("Yes")
    NO = "no", _("No")


class PlanDefinitionActionCardinalityBehavior(models.TextChoices):
    """PlanDefinition action cardinality behavior choices."""

    SINGLE = "single", _("Single")
    MULTIPLE = "multiple", _("Multiple")


class PlanDefinitionActionParticipantType(models.TextChoices):
    """PlanDefinition action participant type choices."""

    CARE_TEAM = "care-team", _("Care Team")
    DEVICE = "device", _("Device")
    GROUP = "group", _("Group")
    HEALTHCARE_SERVICE = "healthcareservice", _("Healthcare Service")
    LOCATION = "location", _("Location")
    ORGANIZATION = "organization", _("Organization")
    PATIENT = "patient", _("Patient")
    PRACTITIONER = "practitioner", _("Practitioner")
    PRACTITIONER_ROLE = "practitionerrole", _("Practitioner Role")
    RELATED_PERSON = "relatedperson", _("Related Person")


class PlanDefinitionActionRelatedActionRelationship(models.TextChoices):
    """PlanDefinition action related action relationship choices."""

    BEFORE_START = "before-start", _("Before Start")
    BEFORE = "before", _("Before")
    BEFORE_END = "before-end", _("Before End")
    CONCURRENT_WITH_START = "concurrent-with-start", _("Concurrent with Start")
    CONCURRENT = "concurrent", _("Concurrent")
    CONCURRENT_WITH_END = "concurrent-with-end", _("Concurrent with End")
    AFTER_START = "after-start", _("After Start")
    AFTER = "after", _("After")
    AFTER_END = "after-end", _("After End")


class PlanDefinitionActionConditionKind(models.TextChoices):
    """PlanDefinition action condition kind choices."""

    APPLICABILITY = "applicability", _("Applicability")
    START = "start", _("Start")
    STOP = "stop", _("Stop")



"""Related persons serializers."""

from dfhir.base.serializers import (
    AddressSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CommunicationSerializer,
    ContactPointSerializer,
    HumanNameSerializer,
    IdentifierSerializer,
    PeriodSerializer,
)
from dfhir.patients.serializers import PatientReferenceSerializer

from .models import RelatedPerson, RelatedPersonReference


class RelatedPersonReferenceSerializer(BaseReferenceModelSerializer):
    """Related person reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = RelatedPersonReference
        exclude = ["created_at", "updated_at"]


class RelatedPersonSerializer(BaseWritableNestedModelSerializer):
    """Related person serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    relationship = CodeableConceptSerializer(many=True, required=False)
    patient = PatientReferenceSerializer(required=False)
    role = CodeableConceptSerializer(many=True, required=False)
    name = HumanNameSerializer(many=True, required=False)
    telecom = ContactPointSerializer(many=True, required=False)
    address = AddressSerializer(many=True, required=False)
    photo = AttachmentSerializer(many=True, required=False)
    period = PeriodSerializer(required=False)
    communication = CommunicationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = RelatedPerson
        exclude = ["created_at", "updated_at"]



"""transport urls."""

from django.urls import path

from dfhir.transports import views

app_name = "transports"

urlpatterns = [
    path("transports/", views.TransportListView.as_view(), name="transport-list"),
    path(
        "transports/<int:pk>/",
        views.TransportDetailView.as_view(),
        name="transport-detail",
    ),
]



# Generated by Django 5.2 on 2025-05-05 14:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('biologicallyderivedproducts', '0002_initial'),
        ('documentreferences', '0005_initial'),
        ('genomicstudies', '0003_initial'),
        ('imagingselections', '0003_initial'),
        ('imagingstudies', '0002_initial'),
        ('immunizations', '0003_initial'),
        ('medicationadministrations', '0004_initial'),
        ('medicationdispenses', '0003_initial'),
        ('medicationstatements', '0004_initial'),
        ('molecularsequences', '0001_initial'),
        ('nutritionorders', '0004_nutritionorder_allergy_intolerance'),
        ('nutritionproducts', '0001_initial'),
        ('observations', '0003_initial'),
        ('procedures', '0002_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='documentreferenceobservationreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='document_reference_observation_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='observationbasedonreference',
            name='nutrition_order',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_based_on_reference_nutrition_order', to='nutritionorders.nutritionorder'),
        ),
        migrations.AddField(
            model_name='observationderivedfromreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_derived_from_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='observationderivedfromreference',
            name='imaging_selection',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='imagingselections.imagingselection'),
        ),
        migrations.AddField(
            model_name='observationderivedfromreference',
            name='imaging_study',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='imagingstudies.imagingstudy'),
        ),
        migrations.AddField(
            model_name='observationderivedfromreference',
            name='molecular_sequence',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_derived_from_reference_molecular_sequence', to='molecularsequences.molecularsequence'),
        ),
        migrations.AddField(
            model_name='observationhasmemberreference',
            name='molecular_sequence',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_has_member_reference_molecular_sequence', to='molecularsequences.molecularsequence'),
        ),
        migrations.AddField(
            model_name='observationpartofreference',
            name='genomic_study',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_part_of_reference_genomic_study', to='genomicstudies.genomicstudy'),
        ),
        migrations.AddField(
            model_name='observationpartofreference',
            name='imaging_study',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_part_of_reference_imaging_study', to='imagingstudies.imagingstudy'),
        ),
        migrations.AddField(
            model_name='observationpartofreference',
            name='immunization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_part_of_reference_immunization', to='immunizations.immunization'),
        ),
        migrations.AddField(
            model_name='observationpartofreference',
            name='medication_administration',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_part_of_reference_medication_administration', to='medicationadministrations.medicationadministration'),
        ),
        migrations.AddField(
            model_name='observationpartofreference',
            name='medication_dispense',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_part_of_reference_medication_dispense', to='medicationdispenses.medicationdispense'),
        ),
        migrations.AddField(
            model_name='observationpartofreference',
            name='medication_statement',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_part_of_reference_medication_statement', to='medicationstatements.medicationstatement'),
        ),
        migrations.AddField(
            model_name='observationpartofreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_part_of_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='observationsubjectreference',
            name='biologically_derived_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_subject_reference_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct'),
        ),
        migrations.AddField(
            model_name='observationsubjectreference',
            name='nutrition_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_subject_reference_nutrition_product', to='nutritionproducts.nutritionproduct'),
        ),
        migrations.AddField(
            model_name='observationsubjectreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_subject_reference_procedure', to='procedures.procedure'),
        ),
    ]



"""Substances views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Substance
from .serializers import SubstanceSerializer


class SubstanceListView(APIView):
    """Substance list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: SubstanceSerializer(many=True)})
    def get(self, request):
        """Get substance list."""
        substances = Substance.objects.all()
        serializer = SubstanceSerializer(substances, many=True)
        return Response(serializer.data)

    @extend_schema(request=SubstanceSerializer, responses={201: SubstanceSerializer})
    def post(self, request):
        """Create a substance."""
        serializer = SubstanceSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class SubstanceDetailView(APIView):
    """Substance detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get substance object."""
        try:
            return Substance.objects.get(pk=pk)
        except Substance.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: SubstanceSerializer})
    def get(self, request, pk=None):
        """Get substance detail."""
        substance = self.get_object(pk)
        serializer = SubstanceSerializer(substance)
        return Response(serializer.data)

    @extend_schema(request=SubstanceSerializer, responses={200: SubstanceSerializer})
    def patch(self, request, pk=None):
        """Update substance."""
        substance = self.get_object(pk)
        serializer = SubstanceSerializer(substance, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a substance."""
        substance = self.get_object(pk)
        substance.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Specimen definitions views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import SpecimenDefinition
from .serializers import SpecimenDefinitionSerializer


class SpecimenDefinitionListView(APIView):
    """Specimen Definition List Views."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200, SpecimenDefinitionSerializer(many=True)})
    def get(self, request):
        """Get all specimen definitions."""
        specimen_definitions = SpecimenDefinition.objects.all()
        serializer = SpecimenDefinitionSerializer(specimen_definitions, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=SpecimenDefinitionSerializer,
        responses={201, SpecimenDefinitionSerializer},
    )
    def post(self, request):
        """Create a specimen definition."""
        serializer = SpecimenDefinitionSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class SpecimenDefinitionDetailView(APIView):
    """Specimen Definition Detail View."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get specimen definition object."""
        try:
            return SpecimenDefinition.objects.get(pk=pk)
        except SpecimenDefinition.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200, SpecimenDefinitionSerializer})
    def get(self, request, pk=None):
        """Get specimen definition detail."""
        specimen_definition = self.get_object(pk)
        serializer = SpecimenDefinitionSerializer(specimen_definition)
        return Response(serializer.data)

    @extend_schema(
        request=SpecimenDefinitionSerializer,
        responses={200, SpecimenDefinitionSerializer},
    )
    def patch(self, request, pk=None):
        """Update specimen definition."""
        specimen_definition = self.get_object(pk)
        serializer = SpecimenDefinitionSerializer(
            specimen_definition, data=request.data
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={200, SpecimenDefinitionSerializer})
    def delete(self, request, pk=None):
        """Delete specimen definition."""
        specimen_definition = self.get_object(pk)
        specimen_definition.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""medication requests views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.generics import ListAPIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.medicationrequests.models import (
    AdditionalIllustration,
    DosageMethod,
    DosageRoute,
    DosageSite,
    MedicationRequest,
    MedicationRequestCategory,
    MedicationRequestMedicationCode,
    MedicationRequestReason,
    MedicationRequestReferenceType,
    ReferenceAsNeededFor,
)
from dfhir.medicationrequests.serializers import (
    AdditionalIllustrationSerializer,
    DosageMethodSerializer,
    DosageRouteSerializer,
    DosageSiteSerializer,
    MediationRequestReferenceTypeSerializer,
    MedicationRequestCategorySerializer,
    MedicationRequestMedicationCodeSerializer,
    MedicationRequestReasonSerializer,
    MedicationRequestSerializer,
    ReferenceAsNeededForSerializer,
)


class MedicationRequestList(APIView):
    """MedicationRequestList."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: MedicationRequestSerializer(many=True)})
    def get(self, request):
        """Get medication requests."""
        medication_requests = MedicationRequest.objects.all()
        serializer = MedicationRequestSerializer(medication_requests, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=MedicationRequestSerializer,
        responses={201: MedicationRequestSerializer},
    )
    def post(self, request):
        """Create medication request."""
        serializer = MedicationRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class MedicationRequestDetail(APIView):
    """medication request detail."""

    def get_object(self, pk):
        """Get medication request object."""
        try:
            return MedicationRequest.objects.get(pk=pk)
        except MedicationRequest.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: MedicationRequestSerializer})
    def get(self, request, pk=None):
        """Get medication request."""
        medication_request = self.get_object(pk)
        serializer = MedicationRequestSerializer(medication_request)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: MedicationRequestSerializer})
    def patch(self, request, pk=None):
        """Update medication request."""
        medication_request = self.get_object(pk)
        serializer = MedicationRequestSerializer(
            medication_request, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete medication request."""
        medication_request = self.get_object(pk)
        medication_request.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


@extend_schema(responses={200: MedicationRequestCategorySerializer(many=True)})
class MedicationRequestCategoryList(ListAPIView):
    """medication request category list."""

    queryset = MedicationRequestCategory.objects.all()
    serializer_class = MedicationRequestCategorySerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationRequestReasonSerializer(many=True)})
class MedicationRequestReasonList(ListAPIView):
    """medication request reason list."""

    queryset = MedicationRequestReason.objects.all()
    serializer_class = MedicationRequestReasonSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationRequestMedicationCodeSerializer(many=True)})
class MedicationRequestMedicationCodeList(ListAPIView):
    """medication request medication code list."""

    queryset = MedicationRequestMedicationCode.objects.all()
    serializer_class = MedicationRequestMedicationCodeSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MediationRequestReferenceTypeSerializer(many=True)})
class MediationRequestReferenceTypeList(ListAPIView):
    """medication request reference type list."""

    queryset = MedicationRequestReferenceType.objects.all()
    serializer_class = MediationRequestReferenceTypeSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationRequestSerializer(many=True)})
class AdditionalIllustrationList(ListAPIView):
    """additional illustration list."""

    queryset = AdditionalIllustration.objects.all()
    serializer_class = AdditionalIllustrationSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationRequestSerializer(many=True)})
class DosageSiteList(ListAPIView):
    """dosage site list."""

    queryset = DosageSite.objects.all()
    serializer_class = DosageSiteSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationRequestSerializer(many=True)})
class DosageRouteList(ListAPIView):
    """dosage route list."""

    queryset = DosageRoute.objects.all()
    serializer_class = DosageRouteSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationRequestSerializer(many=True)})
class DosageMethodList(ListAPIView):
    """dosage method list."""

    queryset = DosageMethod.objects.all()
    serializer_class = DosageMethodSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationRequestSerializer(many=True)})
class ReferenceAsNeededForList(ListAPIView):
    """reference as needed for list."""

    queryset = ReferenceAsNeededFor.objects.all()
    serializer_class = ReferenceAsNeededForSerializer
    permission_classes = [AllowAny]



"""Organization views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.organizations.models import Organization
from dfhir.organizations.serializers import OrganizationSerializer

# print(CreateOrganizationSchema.schema())

DATA = {
    "id": 1,
    "name": "City General Hospital",
    "alias": "CGH",
    "website": "https://www.citygeneralhospital.org",
    "email": "info@citygeneralhospital.org",
    "admin_id": 101,
    "status": "active",
    "active": True,
    "type": "hospital_department",
    # "part_of_id": None,
    "description": "A leading hospital providing comprehensive healthcare services to the city.",
    "location": [
        {
            "id": 10,
            "name": "Main Building",
            "status": "active",
            "operational_status": "occupied",
            "alias": "MB",
            "description": "The main building houses the emergency department and outpatient clinics.",
            "mode": "instance",
            "address": "123 Main Street, Springfield",
            "form": "building",
            "position": '{"latitude": 37.7749, "longitude": -122.4194}',
            # "part_of_id": None,
            "hours_of_operation": '{"Monday-Friday": "8 AM - 8 PM", "Saturday": "9 AM - 5 PM"}',
        },
        {
            "id": 11,
            "name": "Emergency Wing",
            "status": "active",
            "operational_status": "occupied",
            "alias": "EW",
            "description": "Dedicated wing for emergency cases.",
            "mode": "instance",
            "address": "123 Main Street, Springfield",
            "form": "wing",
            "position": '{"latitude": 37.7750, "longitude": -122.4195}',
            "part_of_id": 10,
            "hours_of_operation": '{"24/7": true}',
        },
    ],
    "contact": [
        {
            "id": 1,
            "purpose": "General Inquiries",
            "name": "Main Reception",
            "address": "123 Main Street, Springfield",
            "period": "Monday-Friday: 8 AM - 8 PM",
        },
        {
            "id": 2,
            "purpose": "Emergency Contact",
            "name": "Emergency Services",
            "address": "123 Main Street, Springfield",
            "period": "24/7",
        },
    ],
}


# class OrganizationListView(APIView):
#     """Organization list view."""
#
#     permission_classes = [AllowAny]
#
#     @extend_schema(responses={200: OrganizationSerializer(many=True)})
#     def get(self, request, pk=None):
#         """Get organizations."""
#         queryset = Organization.objects.all()
#         serializer = OrganizationSerializer(queryset, many=True)
#         return Response(serializer.data)
#
#     @extend_schema(
#         request=OrganizationSerializer, responses={200: OrganizationSerializer}
#     )
#     def post(self, request):
#         """Create organization."""
#         request_data = request.data
#         serializer = OrganizationSerializer(data=request_data)
#         serializer.is_valid(raise_exception=True)
#         serializer.save()
#         return Response(serializer.data)


class OrganizationListView(APIView):
    """Organization list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: OrganizationSerializer(many=True)})
    def get(self, request, pk=None):
        """Get organizations."""
        queryset = Organization.objects.all()
        serializer = OrganizationSerializer(queryset, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=OrganizationSerializer, responses={200: OrganizationSerializer}
    )
    def post(self, request):
        """Create organization."""
        request_data = request.data
        serializer = OrganizationSerializer(data=request_data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)


class OrganizationDetailView(APIView):
    """Organization detail view."""

    def get_object(self, pk):
        """Get organization object."""
        try:
            return Organization.objects.get(pk=pk)
        except Organization.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: OrganizationSerializer})
    def get(self, request, pk=None):
        """Get organization."""
        queryset = self.get_object(pk)
        serializer = OrganizationSerializer(queryset)
        return Response(serializer.data)

    @extend_schema(responses={200: OrganizationSerializer})
    def patch(self, request, pk=None):
        """Update organization."""
        queryset = self.get_object(pk)
        serializer = OrganizationSerializer(queryset, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    def delete(self, request, pk=None):
        """Delete organization."""
        organization = self.get_object(pk)
        organization.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AppointmentResponse',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('proposed_new_time', models.BooleanField(null=True)),
                ('start', models.DateTimeField(null=True)),
                ('end', models.DateTimeField(null=True)),
                ('participant_status', models.CharField(choices=[('accepted', 'Accepted'), ('declined', 'Declined'), ('tentative', 'Tentative'), ('needs-action', 'Needs Action'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('comment', models.TextField(null=True)),
                ('recurring', models.BooleanField(null=True)),
                ('occurrence_date', models.DateTimeField(null=True)),
                ('recurrence_id', models.IntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AppointmentResponseActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Specimen definitions App configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class SpecimendefinitionsConfig(AppConfig):
    """Specimen definitions config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.specimendefinitions"
    verbose_name = _("Specimen Definitions")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('allergyintolerances', '0002_initial'),
        ('base', '0003_initial'),
        ('encounters', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='allergyintolerance',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='allergy_intolerance_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='allergy_intolerance_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='onset_age',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_onset_age', to='base.age'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='onset_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_onset_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='onset_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_onset_range', to='base.range'),
        ),
    ]



"""biologically derived product dispenses choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class BiologicallyDerivedProductDispenseStatus(models.TextChoices):
    """Biologically derived product dispense status choices."""

    PREPARATION = "preparation", _("Preparation")
    IN_PERSON = "in-person", _("In Person")
    ALLOCATED = "allocated", _("Allocated")
    ISSUED = "issued", _("Issued")
    UNFULFILLED = "unfulfilled", _("Unfulfilled")
    RETURNED = "returned", _("Returned")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")



"""Groups app configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class GroupsConfig(AppConfig):
    """Groups config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.groups"
    verbose_name = _("Groups")



"""Choices for the base app."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class TelecomUseChoices(models.TextChoices):
    """Telecom use choices."""

    HOME = "home", _("home")
    WORK = "work", _("work")
    TEMP = "temp", _("temp")
    OLD = "old", _("old")
    MOBILE = "mobile", _("mobile")


class AddressUseChoices(models.TextChoices):
    """Address use choices."""

    HOME = "home", _("home")
    WORK = "work", _("work")
    TEMP = "temp", _("temp")
    OLD = "old", _("old")
    BILLING = "billing", _("billing")


class AddressTypeChoices(models.TextChoices):
    """Address type choices."""

    POSTAL = "postal", _("postal")
    PHYSICAL = "physical", _("physical")
    BOTH = "both", _("both")


class GenderChoices(models.TextChoices):
    """Gender choices."""

    MALE = "male", _("male")
    FEMALE = "female", _("female")
    OTHER = "other", _("other")
    UNKNOWN = "unknown", _("unknown")


class ParticipantType(models.TextChoices):
    """Participant types."""

    ADM = "ADM", _("Admitter")
    ATTD = "ATND", _("Attender")
    CALLBCK = "CALLBCK", _("Callback Contact")  # # codespell:ignore
    CON = "CON", _("Consultant")
    DIS = "DIS", _("Discharger")
    EMERGENCY = "emergency", _("Emergency")
    ESCORT = "ESC", _("escort")
    REF = "REF", _("Referrer")
    TRANSLATOR = "translator", _("Translator")


class ParticipantStatus(models.TextChoices):
    """Participant Status."""

    ACCEPTED = "accepted", _("Accepted")
    DECLINED = "declined", _("Declined")
    TENTATIVE = "tentative", _("Tentative")
    NEEDS_ACTION = "needs_action", _("Needs Action")


class TelecomSystemChoices(models.TextChoices):
    """Telecom system choices."""

    PHONE = "phone", _("phone")
    FAX = "fax", _("fax")
    EMAIL = "email", _("email")
    SMS = "sms", _("sms")
    URL = "url", _("url")
    OTHER = "other", _("other")


class ContactPointUseChoices(models.TextChoices):
    """Contact point use choices."""

    HOME = "home", _("home")
    WORK = "work", _("work")
    TEMP = "temp", _("temp")
    OLD = "old", _("old")
    MOBILE = "mobile", _("mobile")


class ContactPointSystemChoices(models.TextChoices):
    """Contact point system choices."""

    PHONE = "phone", _("phone")
    FAX = "fax", _("fax")
    EMAIL = "email", _("email")
    SMS = "sms", _("sms")
    URL = "url", _("url")
    OTHER = "other", _("other")


class HumanNameUseChoices(models.TextChoices):
    """Human name use choices."""

    USUAL = "usual", _("usual")
    OFFICIAL = "official", _("official")
    TEMP = "temp", _("temp")
    NICKNAME = "nickname", _("nickname")
    ANONYMOUS = "anonymous", _("anonymous")
    OLD = "old", _("old")
    MAIDEN = "maiden", _("maiden")


class DaysOfWeekChoices(models.TextChoices):
    """Days of week choices."""

    MON = "mon", _("Monday")
    TUE = "tue", _("Tuesday")
    WED = "wed", _("Wednesday")
    THU = "thu", _("Thursday")
    FRI = "fri", _("Friday")
    SAT = "sat", _("Saturday")
    SUN = "sun", _("Sunday")


class QuantityComparatorChoices(models.TextChoices):
    """Quantity comparator choices."""

    EQUAL = "eq", _("Equal")
    NOT_EQUAL = "ne", _("Not Equal")
    GREATER_THAN = "gt", _("Greater Than")
    LESS_THAN = "lt", _("Less Than")
    GREATER_OR_EQUAL = "ge", _("Greater or Equal")
    LESS_OR_EQUAL = "le", _("Less or Equal")


class ParticipationStatusChoices(models.TextChoices):
    """Participation statuses."""

    ACCEPTED = "accepted", _("Accepted")
    DECLINED = "declined", _("Declined")
    TENTATIVE = "tentative", _("Tentative")
    NEEDSACTION = "needs-action", _("Needs Action")


class RepeatDurationUnits(models.TextChoices):
    """Repeat duration units."""

    SECOND = "s", _("second")
    MINUTE = "min", _("minute")
    HOUR = "h", _("hour")
    DAY = "d", _("day")
    WEEK = "wk", _("week")
    MONTH = "mo", _("month")
    YEAR = "yr", _("year")


class RelatedArtifactTypeChoices(models.TextChoices):
    """Related artifact type choices."""

    DOCUMENTATION = "documentation", _("documentation")
    JUSTIFICATION = "justification", _("justification")
    CITATION = "citation", _("citation")
    PREDECESSOR = "predecessor", _("predecessor")
    SUCCESSOR = "successor", _("successor")
    DERIVED_FROM = "derived-from", _("derived-from")
    DEPENDS_ON = "depends-on", _("depends-on")
    COMPOSED_OF = "composed-of", _("composed-of")
    PART_OF = "part-of", _("part-of")
    REPLACES = "replaces", _("replaces")
    AMENDS = "amends", _("amends")
    APPENDS = "appends", _("appends")
    TRANSFORMS = "transforms", _("transforms")
    AMENDED_WITH = "amended-with", _("amended-with")
    APPENDED_WITH = "appended-with", _("appended-with")
    CITES = "cites", _("cites")
    CITED_BY = "cited-by", _("cited-by")
    COMMENTS_ON = "comments-on", _("comments-on")
    COMMENTS_IN = "comments-in", _("comments-in")
    CONTAINS = "contains", _("contains")
    CONTAINED_IN = "contained-in", _("contained-in")
    CORRECTS = "corrects", _("corrects")
    CORRECTION_IN = "correction-in", _("correction-in")
    REPLACED_WITH = "replaced-with", _("replaced-with")
    RETRACTS = "retracts", _("retracts")
    RETRACTED_BY = "retracted-by", _("retracted-by")
    SIGNS = "signs", _("signs")
    SIMILAR_TO = "similar-to", _("similar-to")
    SUPPORTS = "supports", _("supports")
    SUPPORTED_WITH = "supported-with", _("supported-with")
    TRANSFORMED_INT0 = "transformed-into", _("transformed-into")
    TRANSFORMED_WITH = "transformed-with", _("transformed-with")
    DOCUMENTS = "documents", _("documents")
    SPECIFICATION_OF = "specification-of", _("specification-of")
    CREATED_WITH = "created-with", _("created-with")
    CITE_AS = "cite-as", _("cite-as")


class PublicationStatusChoices(models.TextChoices):
    """Publication status choices."""

    DRAFT = "draft", _("draft")
    ACTIVE = "active", _("active")
    RETIRED = "retired", _("retired")
    UNKNOWN = "unknown", _("unknown")


class MonetaryComponentChoices(models.TextChoices):
    """Monetary component choices."""

    BASE = "base", _("base")
    SURCHARGE = "surcharge", _("surcharge")
    DISCOUNT = "discount", _("discount")
    TAX = "tax", _("tax")
    INFORMATIONAL = "informational", _("informational")


class TriggerDefinitionTypeChoices(models.TextChoices):
    """Trigger definition type choices."""

    NAMED_EVENT = "named-event", _("Named Event")
    PERIODIC = "periodic", _("Periodic")
    DATA_CHANGED = "data-changed", _("Data Changed")
    DATA_ADDED = "data-added", _("Data Added")
    DATA_MODIFIED = "data-modified", _("Data Modified")
    DATA_REMOVED = "data-removed", _("Data Removed")
    DATA_ACCESSED = "data-accessed", _("Data Accessed")
    DATA_ACCESSED_ENDED = "data-accessed-ended", _("Data Accessed Ended")



"""choices module for dfhir observations."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class ObservationStatus(TextChoices):
    """observation status choices."""

    REGISTERED = "registered", _("Registered")
    PRELIMINARY = "preliminary", _("Preliminary")
    FINAL = "final", _("Final")
    AMENDED = "amended", _("Amended")
    CORRECTED = "corrected", _("Corrected")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")


class AbsentReason(TextChoices):
    """absent reason choices."""

    UNKNOWN = "unknown", _("Unknown")
    ASKED_BUT_UNKNOWN = "asked-unknown", _("Asked but unknown")
    TEMPORARILY_UNKNOWN = "temporarily_unknown", _("Temporarily unknown")
    NOT_ASKED = "not_asked", _("Not asked")
    ASKED_BUT_DECLINED = "asked_declined", _("Asked but declined")
    MASKED = "masked", _("Masked")
    NOT_APPLICABLE = "not_applicable", _("Not applicable")
    UNSUPPORTED = "unsupported", _("Unsupported")
    AS_TEXT = "as_text", _("As text")
    ERROR = "error", _("Error")
    NOT_A_NUMBER = "not_a_number", _("Not a number")
    NEGATIVE_INFINITY = "negative_infinity", _("Negative infinity")
    POSITIVE_INFINITY = "positive_infinity", _("Positive infinity")
    NOT_PERFORMED = "not_performed", _("Not performed")
    NOT_PERMITTED = "not_permitted", _("Not permitted")


class ReferenceNormalValues(TextChoices):
    """reference normal values choices."""

    NEGATIVE = "negative", _("Negative")
    ABSENT = "absent", _("Absent")


class ReferenceType(TextChoices):
    """reference type choices."""

    TYPE = "type", _("Type")
    NORMAL_RANGE = "normal_range", _("Normal Range")
    RECOMMENDED_RANGE = "recommended_range", _("Recommended Range")
    TREATMENT_RANGE = "treatment_range", _("Treatment Range")
    THERAPEUTIC_DESIRED_LEVEL = (
        "therapeutic_desired_level",
        _("Therapeutic Desired Level"),
    )
    PRE_THERAPEUTIC_DESIRED_LEVEL = (
        "pre_therapeutic_desired_level",
        _("Pre-Therapeutic Desired Level"),
    )
    POST_THERAPEUTIC_DESIRED_LEVEL = (
        "post_therapeutic_desired_level",
        _("Post-Therapeutic Desired Level"),
    )
    ENDOCRINE = "endocrine", _("Endocrine")
    PRE_PUBERTY = "pre_puberty", _("Pre-Puberty")
    FOLLICULAR_STAGE = "follicular_stage", _("Follicular Stage")
    MID_CYCLE = "mid_cycle", _("Mid-Cycle")
    LUTEAL = "luteal", _("Luteal")
    POST_MENOPAUSE = "post_menopause", _("Post-Menopause")


class TriggeredByType(TextChoices):
    """triggered by type choices."""

    REFLEX = "reflex", _("Reflex")
    REPEAT = "repeat", _("Repeat")
    RE_RUN = "re_run", _("Re-run")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('bodystructures', '0001_initial'),
        ('devices', '0001_initial'),
        ('encounters', '0002_initial'),
        ('observationdefinitions', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ObservationBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationBodyStructureReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationComponent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_integer', models.IntegerField(null=True)),
                ('value_time', models.TimeField(null=True)),
                ('value_date_time', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationDerivedFromReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationHasMemberReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationPartOfReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationReferenceRange',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('text', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationTriggeredBy',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(choices=[('reflex', 'Reflex'), ('repeat', 'Repeat'), ('re_run', 'Re-run')], max_length=255, null=True)),
                ('reason', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SampledData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('interval', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('lower_limit', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('upper_limit', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('dimensions', models.IntegerField()),
                ('offsets', models.CharField(max_length=255, null=True)),
                ('data', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SampledDataIntervalUnitCodes',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('definition', models.TextField(null=True)),
                ('source', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DocumentReferenceObservationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='document_reference_observation_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Observation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('registered', 'Registered'), ('preliminary', 'Preliminary'), ('final', 'Final'), ('amended', 'Amended'), ('corrected', 'Corrected'), ('cancelled', 'Cancelled'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('effective_date_time', models.DateTimeField(null=True)),
                ('effective_instant', models.DateTimeField(null=True)),
                ('issued', models.DateTimeField(null=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_integer', models.IntegerField(null=True)),
                ('value_time', models.TimeField(null=True)),
                ('value_date_time', models.DateTimeField(null=True)),
                ('body_site', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_body_site', to='base.codeableconcept')),
                ('body_structure', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_body_structure', to='bodystructures.bodystructurereference')),
                ('category', models.ManyToManyField(blank=True, related_name='observation_category', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.codeableconcept')),
                ('data_absent_reason', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_data_absent_reason', to='base.codeableconcept')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_device', to='devices.devicedevicemetricreference')),
                ('effective_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_effective_period', to='base.period')),
                ('effective_timing', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_effective_timing', to='base.timing')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='encounters.encounterreference')),
                ('focus', models.ManyToManyField(related_name='observation_focus', to='base.reference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='observation_identifier', to='base.identifier')),
                ('instantiates_reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_instantiates_reference', to='observationdefinitions.observationdefinitionreference')),
                ('interpretation', models.ManyToManyField(blank=True, related_name='observation_interpretation', to='base.codeableconcept')),
                ('method', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_method', to='base.codeableconcept')),
                ('note', models.ManyToManyField(blank=True, related_name='observation_note', to='base.annotation')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""provinces models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    CodeableReference,
    Identifier,
    Reference,
    Signature,
    TimeStampedModel,
)

from .choices import ProvenanceEntryRoleChoices


class ProvenanceAgentWhoReference(BaseReference):
    """provenance agent who reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.PractitionerReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRoleReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "base.OrganizationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeamReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_care_team",
    )
    device = models.ForeignKey(
        "devices.DeviceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_device",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPersonReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_related_person",
    )
    group = models.ForeignKey(
        "groups.GroupReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_group",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareServiceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who_reference_healthcare_service",
    )


class ProvenanceAgentOnBehalfOf(BaseReference):
    """provenance agent on behalf of model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.PractitionerReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRoleReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_practitioner_role",
    )
    organization = models.ForeignKey(
        "base.OrganizationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeamReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_care_team",
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_patient",
    )
    group = models.ForeignKey(
        "groups.GroupReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_group",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareServiceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of_healthcare_service",
    )


class ProvenanceAgent(TimeStampedModel):
    """provenance agent."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_type",
    )
    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_role",
    )
    who = models.ForeignKey(
        ProvenanceAgentWhoReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_who",
    )
    on_behalf_of = models.ForeignKey(
        ProvenanceAgentOnBehalfOf,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_agent_on_behalf_of",
    )


class ProvenanceEntry(TimeStampedModel):
    """provenance entry."""

    role = models.CharField(
        max_length=255,
        null=True,
        choices=ProvenanceEntryRoleChoices.choices,
        default=ProvenanceEntryRoleChoices.REVISION,
    )
    what = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_entry_what",
    )
    agent = models.ManyToManyField(
        ProvenanceAgent, related_name="provenance_entry_agent", blank=True
    )


class Provenance(TimeStampedModel):
    """Provinance model."""

    target = models.ManyToManyField(
        Reference, related_name="provinance_target", blank=True
    )
    occurred_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        related_name="provinance_occurred_period",
        null=True,
    )
    occurred_date_time = models.DateTimeField(null=True)
    recorded = models.DateTimeField(null=True)
    policy = ArrayField(models.URLField(), null=True)
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_location",
    )
    authorization = models.ManyToManyField(
        CodeableReference, related_name="provenance_authorization", blank=True
    )
    why = models.TextField(null=True)
    activity = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_activity",
    )
    based_on = models.ManyToManyField(
        Reference, related_name="provenance_based_on", blank=True
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_patient",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="provenance_encounter",
    )
    agent = models.ManyToManyField(
        ProvenanceAgent, related_name="provenance_agent", blank=True
    )
    entity = models.ManyToManyField(
        ProvenanceEntry, related_name="provenance_entity", blank=True
    )
    signature = models.ManyToManyField(
        Signature, related_name="provenance_signature", blank=True
    )


class ProvenanceReference(BaseReference):
    """medication request insurance reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="provinance_reference_identifier",
    )
    provenance = models.ForeignKey(
        Provenance,
        on_delete=models.SET_NULL,
        null=True,
        related_name="provinance_reference_provenance",
    )



"""medication requests model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class DoseAndRateType(TextChoices):
    """dose type model."""

    CALCULATED = "calculated", _("Calculated")
    ORDERED = "ordered", _("Ordered")


class MedicationRequestStatusReasons(TextChoices):
    """medication request status reason choices."""

    TRY_ANOTHER_TREATMENT_FIRST = (
        "try_another_treatment_first",
        _("Try another treatment first"),
    )
    PRESCRIPTION_REQUIRES_CLARIFICATION = (
        "prescription_requires_clarification",
        _("Prescription requires clarification"),
    )
    DRUG_LEVEL_TOO_HIGH = "drug_level_too_high", _("Drug level too high")
    ADMISSION_TO_HOSPITAL = "admission_to_hospital", _("Admission to hospital")
    LAB_INTERFERENCE_ISSUES = "lab_interference_issues", _("Lab interference issues")
    PATIENT_NOT_AVAILABLE = "patient_not_available", _("Patient not available")
    PATIENT_IS_PREGNANT = "patient_is_pregnant", _("Patient is pregnant")
    PATIENT_IS_BREASTFEEDING = "patient_is_breastfeeding", _("Patient is breastfeeding")
    ALLERGY = "allergy", _("Allergy")
    DRUG_INTERACTS_WITH_ANOTHER_DRUG = (
        "drug_interacts_with_another_drug",
        _("Drug interacts with another drug"),
    )
    DUPLICATE_THERAPY = "duplicate_therapy", _("Duplicate therapy")
    SUSPECTED_INTOLERANCE = "suspected_intolerance", _("suspected intolerance")
    PATIENT_SCHEDULED_FOR_SURGERY = (
        "patient_scheduled_for_surgery",
        _("Patient scheduled for surgery"),
    )
    WAITING_FOR_OLD_DRUG_TO_WASH_OUT = (
        "waiting_for_old_drug_to_wash_out",
        _("Waiting for old drug to wash out"),
    )


class MedicationRequestPriority(TextChoices):
    """medication requests priority."""

    ROUTINE = "routine", _("Routine")
    URGENT = "urgent", _("Urgent")
    ASAP = "asap", _("ASAP")
    STAT = "stat", _("STAT")


class MedicationRequestPerformerType(TextChoices):
    """medication request performer type."""

    REGISTERED_NURSE = "registered_nurse", _("Registered nurse")
    ONCOLOGY_NURSE = "oncology_nurse", _("Oncology nurse")
    PAIN_CONTROL_NURSE = "pain_control_nurse", _("Pain control nurse")
    PHYSICIAN = "physician", _("Physician")
    PHARMACIST = "pharmacist", _("Pharmacist")


class CourseOfTherapyType(TextChoices):
    """course of therapy type."""

    CONTINUOUS_LONG_TERM_THERAPY = (
        "continuous_long_term_therapy",
        _("Continuous long term therapy"),
    )
    SHORT_COURSE_THERAPY = "short_course_therapy", _("Short course therapy")
    SEASONAL = "seasonal", _("Seasonal")


class MedicationRequestStatus(TextChoices):
    """medication request status."""

    ACTIVE = "active", _("Active")
    ON_HOLD = "on_hold", _("On hold")
    ENDED = "ended", _("Ended")
    STOPPED = "stopped", _("Stopped")
    COMPLETED = "completed", _("Completed")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered_in_error", _("Entered in error")
    DRAFT = "draft", _("Draft")
    UNKNOWN = "unknown", _("Unknown")


class MedicationIntent(TextChoices):
    """medication request intent."""

    PLAN = "plan", _("Plan")
    ORDER = "order", _("Order")
    ORIGINAL_ORDER = "original_order", _("Original order")
    FILLER_ORDER = "filler_order", _("Filler order")
    INSTANCE_ORDER = "instance_order", _("Instance order")
    OPTION = "option", _("Option")


class DoseAdministrationAid(TextChoices):
    """dose administration aid."""

    BLISTER_PACK = "blister_pack", _("Blister pack")
    DOSETTE = "dosette", _("Dosette")
    SACHETS = "sachets", _("Sachets")


class SubstitutionReason(TextChoices):
    """substitution reason."""

    CONTINUING_THERAPY = "continuing_therapy", _("Continuing therapy")
    FORMULARY_POLICY = "formulary_policy", _("Formulary policy")
    OUT_OF_STOCK = "out of stock", _("Out of stock")
    REGULATORY_REQUIREMENTS = "regulatory_requirements", _("Regulatory requirements")



"""nutrition intake model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class NutritionIntakeStatusChoices(TextChoices):
    """Nutrition intake status choices."""

    PREPARATION = "preparation", _("Preparation")
    IN_PROGRESS = "in-progress", _("In Progress")
    NOT_DONE = "not-done", _("Not Done")
    ONE_HOLD = "on-hold", _("On Hold")
    STOPPED = "stopped", _("Stopped")
    COMPLETED = "completed", _("Completed")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('devices', '0001_initial'),
        ('encounters', '0002_initial'),
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='FlagSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FlagSupportingInfoReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Flag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'active'), ('inactive', 'inactive'), ('entered-in-error', 'entered-in-error')], max_length=255, null=True)),
                ('category', models.ManyToManyField(blank=True, related_name='flag_category', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_code', to='base.codeableconcept')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_encounter', to='encounters.encounterreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='flag_identifier', to='base.identifier')),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_period', to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='FlagAuthorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_author_reference_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_author_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_author_reference_organization', to='organizations.organization')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Locations app configuration."""

from django.apps import AppConfig


class LocationsConfig(AppConfig):
    """Locations app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.locations"



"""Location views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .filters import LocationFilter
from .models import Location
from .serializers import LocationSerializer


class LocationList(APIView):
    """Location list view."""

    permission_classes = [AllowAny]
    serializer = LocationSerializer
    filter_class = LocationFilter

    @extend_schema(responses={200: LocationSerializer(many=True)}, filters=True)
    def get(self, request):
        """Get locations."""
        locations = Location.objects.all()
        locations_filter = self.filter_class(request.GET, queryset=locations)
        serializer = LocationSerializer(locations_filter.qs, many=True)
        return Response(serializer.data)

    @extend_schema(request=LocationSerializer, responses={200: LocationSerializer})
    def post(self, request):
        """Create location."""
        serializer = LocationSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class LocationDetail(APIView):
    """Location detail view."""

    permission_classes = [AllowAny]
    serializer = LocationSerializer

    def get_object(self, pk):
        """Get location object."""
        try:
            return Location.objects.get(pk=pk)
        except Location.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: LocationSerializer})
    def get(self, request, pk=None):
        """Get location."""
        location = Location.objects.get(pk=pk)
        serializer = self.serializer(location)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: LocationSerializer})
    def patch(self, request, pk=None):
        """Update location."""
        location = Location.objects.get(pk=pk)
        serializer = self.serializer(location, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete location."""
        location = Location.objects.get(pk=pk)
        location.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Adverse events views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import AdverseEvent
from .serializers import AdverseEventSerializer


class AdverseEventListView(APIView):
    """Adverse event list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: AdverseEventSerializer(many=True)})
    def get(self, request):
        """Get a list of adverse events."""
        adverse_events = AdverseEvent.objects.all()
        serializer = AdverseEventSerializer(adverse_events, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=AdverseEventSerializer, responses={201: AdverseEventSerializer}
    )
    def post(self, request):
        """Create an adverse event."""
        serializer = AdverseEventSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data, status=status.HTTP_201_CREATED)


class AdverseEventDetailView(APIView):
    """Adverse event detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get an adverse event object."""
        try:
            return AdverseEvent.objects.get(pk=pk)
        except AdverseEvent.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: AdverseEventSerializer})
    def get(self, request, pk):
        """Get an adverse event."""
        adverse_event = self.get_object(pk)
        serializer = AdverseEventSerializer(adverse_event)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=AdverseEventSerializer, responses={200: AdverseEventSerializer}
    )
    def patch(self, request, pk):
        """Update an adverse event."""
        adverse_event = self.get_object(pk)
        serializer = AdverseEventSerializer(
            adverse_event, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete an adverse event."""
        adverse_event = self.get_object(pk)
        adverse_event.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ActivityDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('url', models.URLField(null=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('version_algorithm_string', models.CharField(max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('subtitle', models.CharField(max_length=255, null=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('active', 'Active'), ('retired', 'Retired'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('experimental', models.BooleanField(null=True)),
                ('date', models.DateTimeField(null=True)),
                ('publisher', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('purpose', models.TextField(null=True)),
                ('usage', models.TextField(null=True)),
                ('copyright', models.TextField(null=True)),
                ('copyright_label', models.TextField(null=True)),
                ('approval_date', models.DateField(null=True)),
                ('last_review_date', models.DateField(null=True)),
                ('intent', models.CharField(choices=[('proposal', 'Proposal'), ('plan', 'Plan'), ('directive', 'Directive'), ('order', 'Order'), ('original-order', 'Original Order'), ('reflex-order', 'Reflex Order'), ('filter-order', 'Filter Order'), ('instance-order', 'Instance Order'), ('option', 'Option')], max_length=255, null=True)),
                ('priority', models.CharField(choices=[('routine', 'Routine'), ('urgency', 'Urgency'), ('asap', 'ASAP'), ('stat', 'STAT')], max_length=255, null=True)),
                ('do_not_perform', models.BooleanField(null=True)),
                ('as_needed_boolean', models.BooleanField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ActivityDefinitionDynamicValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('path', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ActivityDefinitionParticipant',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(choices=[('careteam', 'Care Team'), ('device', 'Device'), ('group', 'Group'), ('healthcareservice', 'Healthcare Service'), ('location', 'Location'), ('organization', 'Organization'), ('patient', 'Patient'), ('practitioner', 'Practitioner'), ('practitionerrole', 'Practitioner Role'), ('relatedperson', 'Related Person')], max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ActivityDefinitionParticipantTypeReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ActivityDefinitionProductProductReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ActivityDefinitionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ActivityDefinitionSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0003_initial'),
        ('base', '0003_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='InvoiceLineItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField(null=True)),
                ('service_date', models.DateField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InvoiceParticipant',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InvoiceParticipantActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InvoiceRecipientReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('issued', 'Issued'), ('balanced', 'Balanced'), ('cancelled', 'Cancelled'), ('entered_in_error', 'Entered in error')], default='draft', max_length=255)),
                ('cancelled_reason', models.TextField(null=True)),
                ('date', models.DateTimeField(null=True)),
                ('creation', models.DateTimeField(null=True)),
                ('period_date', models.DateField(null=True)),
                ('payment_terms', models.TextField(null=True)),
                ('account', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_account', to='accounts.accountreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='invoice_identifier', to='base.identifier')),
                ('issuer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_issuer', to='base.organizationreference')),
                ('note', models.ManyToManyField(blank=True, related_name='invoice_note', to='base.annotation')),
                ('period_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_period_period', to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('careplans', '0004_initial'),
        ('communications', '0002_initial'),
        ('devicerequests', '0002_initial'),
        ('immunizationrecommendations', '0002_initial'),
        ('medicationrequests', '0001_initial'),
        ('nutritionorders', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='communication',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='communication',
            name='topic',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_topic', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='communicationbsedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_based_on_reference_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='communicationbsedonreference',
            name='device_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_based_on_reference_device_request', to='devicerequests.devicerequest'),
        ),
        migrations.AddField(
            model_name='communicationbsedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_based_on_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='communicationbsedonreference',
            name='immunization_recommendation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_based_on_reference_immunization_recommendation', to='immunizationrecommendations.immunizationrecommendation'),
        ),
        migrations.AddField(
            model_name='communicationbsedonreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_based_on_reference_medication_request', to='medicationrequests.medicationrequest'),
        ),
        migrations.AddField(
            model_name='communicationbsedonreference',
            name='nutrition_order',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_based_on_reference_nutrition_order', to='nutritionorders.nutritionorder'),
        ),
    ]



"""genomic study urls."""

from django.urls import path

from dfhir.genomicstudies import views

app_name = "genomicstudy"

urlpatterns = [
    path("genomicstudy/", views.GenomicStudyListView.as_view()),
    path("genomicstudy/<int:pk>/", views.GenomicStudyDetailView.as_view()),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('biologicallyderivedproducts', '0001_initial'),
        ('devices', '0001_initial'),
        ('groups', '0002_initial'),
        ('locations', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('servicerequests', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SpecimenCollectorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collector_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collector_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collector_reference_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collector_reference_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collector_reference_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collector_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SpecimenCollection',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('collected_date', models.DateTimeField(null=True)),
                ('collected_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_collected_period', to='base.period')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_device', to='devices.devicecodeablereference')),
                ('duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_duration', to='base.duration')),
                ('fasting_status_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_fasting_status_codeable_concept', to='base.codeableconcept')),
                ('fasting_status_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_fasting_status_duration', to='base.duration')),
                ('method', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_method', to='base.codeableconcept')),
                ('procedure', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_procedure', to='procedures.procedurereference')),
                ('quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_method', to='base.simplequantity')),
                ('collector', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections_collector', to='specimens.specimencollectorreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SpecimenContainer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_container_device', to='devices.devicereference')),
                ('specimen_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_container_specimen_quantity', to='base.simplequantity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SpecimenFeature',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.TextField(null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_feature_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SpecimenProcessing',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.TextField(null=True)),
                ('time_date_time', models.DateTimeField(null=True)),
                ('additive', models.ManyToManyField(blank=True, related_name='specimen_processing_additive', to='substances.substancereference')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_processing_device', to='devices.devicereference')),
                ('method', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_processing_method', to='base.codeableconcept')),
                ('time_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_processing_time_duration', to='base.duration')),
                ('time_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_processing_time_period', to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Specimen',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('available', 'Available'), ('unavailable', 'Unavailable'), ('unsatisfactory', 'Unsatisfactory'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('received_time', models.DateTimeField(null=True)),
                ('combined', models.CharField(choices=[('grouped', 'Grouped'), ('pooled', 'Pooled')], max_length=255, null=True)),
                ('accession_identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_accession_identifier', to='base.identifier')),
                ('condition', models.ManyToManyField(blank=True, related_name='specimen_condition', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='specimen_identifier', to='base.identifier')),
                ('note', models.ManyToManyField(blank=True, related_name='specimen_note', to='base.annotation')),
                ('request', models.ManyToManyField(blank=True, related_name='specimen_request', to='servicerequests.servicerequestreference')),
                ('role', models.ManyToManyField(blank=True, related_name='specimen_role', to='base.codeableconcept')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_type', to='base.codeableconcept')),
                ('collections', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_collections', to='specimens.specimencollection')),
                ('container', models.ManyToManyField(blank=True, related_name='specimen_container', to='specimens.specimencontainer')),
                ('feature', models.ManyToManyField(blank=True, related_name='specimen_feature', to='specimens.specimenfeature')),
                ('processing', models.ManyToManyField(blank=True, related_name='specimen_processing', to='specimens.specimenprocessing')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SpecimenProcessingPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_performer_reference_identifier', to='base.identifier')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_performer_reference_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_performer_reference_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_performer_reference_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_performer_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='specimenprocessing',
            name='performer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_processing_performer', to='specimens.specimenprocessingperformerreference'),
        ),
        migrations.CreateModel(
            name='SpecimenReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_reference_identifier', to='base.identifier')),
                ('specimen', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_reference_specimen', to='specimens.specimen')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='specimen',
            name='parent',
            field=models.ManyToManyField(blank=True, related_name='specimen_parent', to='specimens.specimenreference'),
        ),
        migrations.CreateModel(
            name='SpecimenSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('biologically_driven_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject_reference_biologically_driven_product', to='biologicallyderivedproducts.biologicallyderivedproduct')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject_reference_device', to='devices.device')),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject_reference_group', to='groups.group')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject_reference_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject_reference_location', to='locations.location')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject_reference_patient', to='patients.patient')),
                ('substance', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject_reference_substance', to='substances.substance')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='specimen',
            name='subject',
            field=models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='specimen_subject', to='specimens.specimensubjectreference'),
        ),
    ]



"""transport views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.transports.models import Transport
from dfhir.transports.serializers import TransportSerializer


class TransportListView(APIView):
    """TransportListView."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: TransportSerializer(many=True)})
    def get(self, request):
        """Get."""
        transports = Transport.objects.all()
        serializer = TransportSerializer(transports, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=TransportSerializer, responses={201: TransportSerializer})
    def post(self, request):
        """Post."""
        serializer = TransportSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class TransportDetailView(APIView):
    """TransportDetailView."""

    def get_object(self, pk):
        """get_object."""
        try:
            return Transport.objects.get(pk=pk)
        except Transport.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: TransportSerializer})
    def get(self, request, pk):
        """Get."""
        transport = self.get_object(pk)
        serializer = TransportSerializer(transport)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=TransportSerializer, responses={200: TransportSerializer})
    def patch(self, request, pk):
        """Put."""
        transport = self.get_object(pk)
        serializer = TransportSerializer(transport, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete."""
        transport = self.get_object(pk)
        transport.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""nutrition orders models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Reference,
    TimeStampedModel,
    Timing,
)
from dfhir.devicedefinitions.models import DeviceDefinitionCodeableReference
from dfhir.nutritionorders.choices import (
    NutritionOrderIntentChoices,
    NutritionOrderPriorityChoices,
    NutritionOrderStatusChoices,
)
from dfhir.patients.models import PatientGroupReference
from dfhir.practitioners.models import PractitionerPractitionerRoleReference


class NutritionOrderReference(BaseReference):
    """nutrition order reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_reference_identifier",
    )
    nutrition_order = models.ForeignKey(
        "NutritionOrder",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_reference_nutrition_order",
    )


class NutritionOrderCodeableReference(TimeStampedModel):
    """nutrition order codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        NutritionOrderReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_codeable_reference_reference",
    )


class NutritionOrderSchedule(TimeStampedModel):
    """nutrition order oral diet schedule model."""

    timing = models.ManyToManyField(
        Timing,
        related_name="nutrition_order_oral_diet_schedule_timing",
        blank=True,
    )
    as_needed = models.BooleanField(default=False)
    as_needed_for = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        related_name="nutrition_order_oral_diet_schedule_as_needed_for",
        null=True,
    )


class NutritionOrderOralDietNutrient(TimeStampedModel):
    """nutrition order oral diet nutrient model."""

    modifier = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        related_name="nutrition_order_oral_diet_nutrient_modifier",
        null=True,
    )
    amount = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_oral_diet_nutrient_amount",
        null=True,
    )


class NutritionOrderOralDietTexture(TimeStampedModel):
    """nutrition order oral diet texture model."""

    modifier = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        related_name="nutrition_order_oral_diet_texture_modifier",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        related_name="nutrition_order_oral_diet_texture_food_type",
        null=True,
    )


class NutritionOrderOralDiet(TimeStampedModel):
    """nutrition order oral diet model."""

    type = models.ManyToManyField(
        CodeableConcept, related_name="nutrition_order_oral_diet_type"
    )
    schedule = models.ForeignKey(
        NutritionOrderSchedule,
        related_name="nutrition_order_oral_diet_schedule",
        null=True,
        on_delete=models.DO_NOTHING,
    )
    nutrient = models.ManyToManyField(
        NutritionOrderOralDietNutrient,
        related_name="nutrition_order_oral_diet_nutrient",
        blank=True,
    )
    texture = models.ManyToManyField(
        NutritionOrderOralDietTexture,
        related_name="nutrition_order_oral_diet_texture",
        blank=True,
    )
    instruction = models.TextField(blank=True)
    caloric_density = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_oral_diet_caloric_density",
        null=True,
    )


class NutritionOrderSupplement(TimeStampedModel):
    """nutrition order supplement model."""

    type = models.ForeignKey(
        "nutritionproducts.NutritionProductCodeableReference",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_supplement_type",
        null=True,
    )
    productName = models.CharField(max_length=255, blank=True)
    schedule = models.ForeignKey(
        NutritionOrderSchedule,
        related_name="nutrition_order_supplement_schedule",
        null=True,
        on_delete=models.DO_NOTHING,
    )
    quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_supplement_quantity",
        null=True,
    )
    instruction = models.TextField(blank=True)
    caloric_density = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_supplement_caloric_density",
    )


class NutritionOrderEnteralFormulaAdministration(TimeStampedModel):
    """nutrition order enteral formula administration model."""

    schedule = models.ForeignKey(
        NutritionOrderSchedule,
        on_delete=models.SET_NULL,
        related_name="nutrition_order_enternal_formula_administration_schedule",
        null=True,
    )
    quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_enternal_formula_administration_quantity",
        null=True,
    )
    rate_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_enternal_formula_administration_rate_quantity",
        null=True,
    )
    rate_ratio = models.ForeignKey(
        "base.Ratio",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_enternal_formula_administration_rate_ratio",
        null=True,
    )


class NutritionOrderEnteralFormula(TimeStampedModel):
    """nutrition order enteral formula model."""

    type = models.ForeignKey(
        "nutritionproducts.NutritionProductCodeableReference",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_enternal_formula_type",
        null=True,
    )
    product_name = models.CharField(max_length=255, null=True)
    delivery_Device = models.ManyToManyField(
        DeviceDefinitionCodeableReference,
        related_name="nutrition_order_enternal_formula_delivery_device",
        blank=True,
    )
    caloric_density = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        related_name="nutrition_order_enternal_formula_caloric_density",
        null=True,
    )
    route_of_administration = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="nutrition_order_enternal_formula_route_of_administration",
    )
    administration = models.ManyToManyField(
        NutritionOrderEnteralFormulaAdministration,
        related_name="nutrition_order_enternal_formula_administration",
        blank=True,
    )
    max_volume_to_administer = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_enternal_formula_max_volume_to_administer",
    )
    administration_instruction = models.TextField(null=True)


class NutritionOrderAdditive(TimeStampedModel):
    """nutrition order additive model."""

    modular_type = models.ForeignKey(
        "nutritionproducts.NutritionProductCodeableReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_additive_modular_type",
    )
    product_name = models.CharField(max_length=255, blank=True)
    quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_additive_quantity",
    )


class NutritionOrderBasedOnReference(BaseReference):
    """nutrition order based on reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_based_on_reference_identifier",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_based_on_reference_care_plan",
    )
    nutrition_order = models.ForeignKey(
        "NutritionOrder",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_based_on_reference_nutrition_order",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_based_on_reference_service_request",
    )
    # TODO: request_orchestration = models.ForeignKey(
    #     "requestorchestrations.RequestOrchestration",
    #     on_delete=models.SET_NULL,
    #     null=True,
    #     related_name="nutrition_order_based_on_reference_request_orchestration",
    # )


class NutritionOrderPerformerReference(BaseReference):
    """nutrition order performer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_identifier",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_care_team",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_related_person",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_patient",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_organization",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.SET_NULL,
        null=True,
        related_name="nutrition_order_performer_reference_group",
    )


class NutritionOrder(TimeStampedModel):
    """Nutrition Order model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="nutrition_order_identifier"
    )
    # TODO: instantiates_canonical = models.ManyToManyField(ActivityDefintionPlanDefinitionCanonicalReference)
    instantiates_uri = models.URLField(null=True)
    instantiates = models.URLField(null=True)
    based_on = models.ManyToManyField(
        NutritionOrderBasedOnReference,
        blank=True,
        related_name="nutrition_order_based_on",
    )
    group_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="nutrition_order_group_identifier",
    )
    status = models.CharField(
        max_length=255, blank=True, choices=NutritionOrderStatusChoices.choices
    )
    intent = models.CharField(
        max_length=255, null=True, choices=NutritionOrderIntentChoices.choices
    )
    priority = models.CharField(
        max_length=255, null=True, choices=NutritionOrderPriorityChoices.choices
    )
    subject = models.ForeignKey(
        PatientGroupReference,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_order_subject",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        related_name="nutrition_order_encounter",
        null=True,
    )
    supporting_information = models.ManyToManyField(
        Reference, related_name="nutrition_order_supporting_information", blank=True
    )
    date_time = models.DateTimeField(null=True)
    orderer = models.ForeignKey(
        PractitionerPractitionerRoleReference,
        on_delete=models.DO_NOTHING,
        related_name="nutrition_order_orderer",
        null=True,
    )
    performer = models.ManyToManyField(
        NutritionOrderPerformerReference,
        blank=True,
        related_name="nutrition_order_performer",
    )
    allergy_intolerance = models.ManyToManyField(
        "allergyintolerances.AllergyIntoleranceReference", blank=True
    )
    food_preference_modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="nutrition_order_food_preference_modifier",
        blank=True,
    )
    exclude_food_modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="nutrition_order_exclude_food_modifier",
        blank=True,
    )
    outside_food_allowed = models.BooleanField(default=False)
    oral_diet = models.ForeignKey(
        NutritionOrderOralDiet,
        on_delete=models.SET_NULL,
        related_name="nutrition_order_oral_diet",
        null=True,
    )
    supplement = models.ManyToManyField(
        NutritionOrderSupplement,
        related_name="nutrition_order_supplement",
        blank=True,
    )
    enteral_formula = models.ForeignKey(
        NutritionOrderEnteralFormula,
        on_delete=models.SET_NULL,
        related_name="nutrition_order_enteral_formula",
        null=True,
    )
    additive = models.ManyToManyField(
        NutritionOrderAdditive, related_name="nutrition_order_additive", blank=True
    )
    note = models.ManyToManyField(
        Annotation, related_name="nutrition_order_note", blank=True
    )



"""Personal Relationships serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
)
from dfhir.groups.serializers import GroupReferenceSerializer

from .models import (
    PersonalRelationship,
    PersonalRelationshipAsserterReference,
    PersonalRelationshipSourceReference,
    PersonalRelationshipTargetReference,
)


class PersonalRelationshipSourceReferenceSerializer(BaseReferenceModelSerializer):
    """Personal Relationship Source Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = PersonalRelationshipSourceReference
        exclude = ["created_at", "updated_at"]


class PersonalRelationshipTargetReferenceSerializer(BaseReferenceModelSerializer):
    """Personal Relationship Source Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = PersonalRelationshipTargetReference
        exclude = ["created_at", "updated_at"]


class PersonalRelationshipAsserterReferenceSerializer(BaseReferenceModelSerializer):
    """Personal Relationship Asserter Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = PersonalRelationshipAsserterReference
        exclude = ["created_at", "updated_at"]


class PersonalRelationshipSerializer(WritableNestedModelSerializer):
    """Personal Relationship serializer."""

    source = PersonalRelationshipSourceReferenceSerializer(required=False)
    target = PersonalRelationshipTargetReferenceSerializer(required=False)
    asserter = PersonalRelationshipAsserterReferenceSerializer(required=False)
    relationship_type = CodeableConceptSerializer(required=False)
    confidence = CodeableConceptSerializer(required=False)
    period = PeriodSerializer(many=True, required=False)
    group = GroupReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = PersonalRelationship
        exclude = ["created_at", "updated_at"]



"""biologically derived product dispenses urls."""

from django.urls import path

from dfhir.biologicallyderivedproductdispenses import views

app_name = "biologicallyderivedproductdispenses"


urlpatterns = [
    path(
        "biologicallyderivedproductdispenses/",
        views.BiologicallyDerivedProductDispenseListView.as_view(),
    ),
    path(
        "biologicallyderivedproductdispenses/<int:pk>/",
        views.BiologicallyDerivedProductDispenseDetailView.as_view(),
    ),
]



"""charge item serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.accounts.serializers import AccountReferenceSerializer
from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    MonetaryComponentSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    QuantitySerializer,
    ReferenceSerializer,
    TimingSerializer,
)
from dfhir.bodystructures.serializers import BodyStructureReferenceSerializer
from dfhir.chargeitems.models import (
    ChargeItem,
    ChargeItemEntererReference,
    ChargeItemPerformer,
    ChargeItemPerformerActorReference,
    ChargeItemProductCodeableReference,
    ChargeItemProductReference,
    ChargeItemReasonCodeableReference,
    ChargeItemReasonReference,
    ChargeItemReference,
    ChargeItemServiceCodealbeReference,
    ChargeItemServiceReference,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer


class ChargeItemReferenceSerializer(BaseReferenceModelSerializer):
    """Charge item reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ChargeItemReference
        exclude = ["created_at", "updated_at"]


class ChargeItemPerformerActorReferenceSerializer(BaseReferenceModelSerializer):
    """Charge item performer actor reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta  options."""

        model = ChargeItemPerformerActorReference
        exclude = ["created_at", "updated_at"]


class ChargeItemPerformerSerializer(WritableNestedModelSerializer):
    """charge item performer serializer."""

    function = CodeableConceptSerializer(many=False, required=False)
    actor = ChargeItemPerformerActorReferenceSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ChargeItemPerformer
        exclude = ["created_at", "updated_at"]


class ChargeItemEntererReferenceSerializer(BaseReferenceModelSerializer):
    """Charge item enterer reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ChargeItemEntererReference
        exclude = ["created_at", "updated_at"]


class ChargeItemsServiceReference(BaseReferenceModelSerializer):
    """Charge item service reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ChargeItemServiceReference
        exclude = ["created_at", "updated_at"]


class ChargeItemProductReferenceSerializer(BaseReferenceModelSerializer):
    """Charge item product reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ChargeItemProductReference
        exclude = ["created_at", "updated_at"]


class ChargeItemProductCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Charge item product codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = ChargeItemProductReferenceSerializer(required=False)

    class Meta:
        """Meta options."""

        model = ChargeItemProductCodeableReference
        exclude = ["created_at", "updated_at"]


class ChargeItemReasonReferenceSerializer(BaseReferenceModelSerializer):
    """Charge item reason reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ChargeItemReasonReference
        exclude = ["created_at", "updated_at"]


class ChargeItemReasonCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Charge item reason codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = ChargeItemReasonReferenceSerializer(required=False)

    class Meta:
        """Meta options."""

        model = ChargeItemReasonCodeableReference
        exclude = ["created_at", "updated_at"]


class ChargeItemServiceCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Charge item service codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = ChargeItemsServiceReference(required=False)

    class Meta:
        """Meta options."""

        model = ChargeItemServiceCodealbeReference
        exclude = ["created_at", "updated_at"]


class ChargeItemSerializer(BaseWritableNestedModelSerializer):
    """charge item serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    part_of = ChargeItemReferenceSerializer(many=False, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    subject = PatientGroupReferenceSerializer(required=False)
    encounter = EncounterReferenceSerializer(required=False)
    occurrence_period = PeriodSerializer(required=False)
    occurrence_timing = TimingSerializer(required=False)
    performer = ChargeItemPerformerSerializer(many=True, required=False)
    performing_organization = OrganizationReferenceSerializer(required=False)
    requesting_organization = OrganizationReferenceSerializer(required=False)
    cost_center = OrganizationReferenceSerializer(required=False)
    quantity = QuantitySerializer(required=False)
    body_site = BodyStructureReferenceSerializer(many=False, required=False)
    unit_price_component = MonetaryComponentSerializer(many=False, required=False)
    total_price_component = MonetaryComponentSerializer(many=False, required=False)
    override_reason = CodeableConceptSerializer(required=False)
    enterer = ChargeItemEntererReferenceSerializer(required=False)
    reason = ChargeItemReasonCodeableReferenceSerializer(many=True, required=False)
    service = ChargeItemServiceCodeableReferenceSerializer(many=True, required=False)
    product = ChargeItemProductCodeableReferenceSerializer(many=True, required=False)
    account = AccountReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    supporting_information = ReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta options."""

        model = ChargeItem
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('careteams', '0002_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='careteam',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_teams_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='careteam',
            name='telecom',
            field=models.ManyToManyField(blank=True, related_name='care_teams_telecom', to='base.contactpoint'),
        ),
        migrations.AddField(
            model_name='careteamparticipant',
            name='effective_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_participants_effective_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='careteamparticipant',
            name='effective_timing',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_participants_effective_timing', to='base.timing'),
        ),
        migrations.AddField(
            model_name='careteamparticipant',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_participants_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='careteam',
            name='participant',
            field=models.ManyToManyField(blank=True, related_name='care_teams_participant', to='careteams.careteamparticipant'),
        ),
        migrations.CreateModel(
            name='CareTeamParticipantMemberReference',
            fields=[
                ('careteamparticipantonbehalfofreference_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='careteams.careteamparticipantonbehalfofreference')),
            ],
            options={
                'abstract': False,
            },
            bases=('careteams.careteamparticipantonbehalfofreference',),
        ),
        migrations.AddField(
            model_name='careteamparticipantonbehalfofreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='careteamparticipantonbehalfofreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_participant_on_behalf_of_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='careteamparticipantonbehalfofreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_participant_on_behalf_of_reference_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='careteamparticipantonbehalfofreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='patients.patient'),
        ),
    ]
